<?php

declare(strict_types=1);
namespace Drupal\sessionless_forms;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Security\TrustedCallbackInterface;
use Drupal\sessionless\WebTokenService\WebTokenServiceInterface;

final class SessionlessFormLazyBuilder implements TrustedCallbackInterface {

  public function __construct(
    protected WebTokenServiceInterface $webTokenService
  ) {}

  public function lazyBuilder(string|null $serializedForm, string|null $serializedFormState, int $test): array {
    $build = [
      '#cache' => ['max-age' => 0],
    ];
    if ($serializedForm) {
      $build['form'] = [
        '#type' => 'hidden',
        '#name' => 'sessionless_forms__form',
        '#value' => $this->webTokenService->dump($serializedForm),
      ];
    }
    if ($serializedForm) {
      $build['form_state'] = [
        '#type' => 'hidden',
        '#name' => 'sessionless_forms__form_state',
        '#value' => $this->webTokenService->dump($serializedFormState),
      ];
    }
    return $build;
  }

  public static function hookFormAlter(array &$form, FormStateInterface $form_state, $form_id) {
    // Add the lazy builder for our hidden fields.
    $form['sessionless_forms'] = [
      '#lazy_builder' => [
        'sessionless_forms.lazy_builder:lazyBuilder',
        [NULL, NULL, 7],
      ],
      '#create_placeholder' => TRUE,
    ];
    // Store a ref to an alterable form.
    // Sorry kittens, this seems the only way to get that currently.
    $storage =& $form_state->getStorage();
    $storage['sessionless__alterable_form'] =& $form;
    $form_state->setStorage($storage);
  }

  public function resetValues(array &$form) {
    $form['sessionless_forms']['#lazy_builder'][1] = [NULL, NULL];
  }

  public function setForm(FormStateInterface $formState, $form) {
    $alterableForm =& $formState->get('sessionless__alterable_form');
    if (!is_array($alterableForm)) {
      throw new \UnexpectedValueException('CompleteForm must be set here.');
    }

    $copiedForm = $form;
    $this->resetValues($copiedForm);
    $serialized = serialize($copiedForm);
    $alterableForm['sessionless_forms']['#lazy_builder'][1][0] = $serialized;
  }

  public function setFormState(FormStateInterface $formState) {
    $alterableForm =& $formState->get('sessionless__alterable_form');
    if (!is_array($alterableForm)) {
      throw new \UnexpectedValueException('CompleteForm must be set here.');
    }

    $serialized = serialize($formState->getCacheableArray());
    $alterableForm['sessionless_forms']['#lazy_builder'][1][1] = $serialized;
  }

  public function getForm(array $userInput): ?array {
    $webToken = $userInput['sessionless_forms__form'] ?? NULL;
    if ($webToken) {
      $serialized = $this->webTokenService->parse($webToken);
      // Unserializing is safe here, as the value is generated in a trusted way,
      // then signed and encrypted.
      return unserialize($serialized);
    }
    return NULL;
  }

  public function getFormStateArray(array $userInput): ?array {
    $webToken = $userInput['sessionless_forms__form_state'] ?? NULL;
    if ($webToken) {
      $serialized = $this->webTokenService->parse($webToken);
      // Unserializing is safe here, as the value is generated in a trusted way,
      // then signed and encrypted.
      return unserialize($serialized);
    }
    return NULL;
  }

  public static function trustedCallbacks() {
    return ['lazyBuilder'];
  }

}
